package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.util.upload.UploadUtil
import com.gxzc.zen.common.util.upload.ZenFileMetadata
import com.gxzc.zen.common.util.upload.constants.CheckStatus
import com.gxzc.zen.common.util.upload.constants.UploadStatus
import com.gxzc.zen.logging.annotation.LogAnnotation
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

/**
 * 上传文件 控制器
 * @author NorthLan
 * @date 2018/5/19
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("/upload")
class UploadController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(UploadController::class.java)
    }

    @ApiOperation("获取已上传分片列表")
    @LogAnnotation(ignore = true)
    @GetMapping
    fun checkChunk(fileMetadata: ZenFileMetadata): ResponseEntity<*> {
        // 检查已上传文件分片
        val ret = UploadUtil.checkUpload(fileMetadata)
        return if (ret.uploadedChunks != null) {
            when (ret.checkStatus) {
            // 不入库
                CheckStatus.FILE_EXISTS,
                CheckStatus.NO_FILE_FULL_CHUNK -> {
                    // 文件存在
                    ret.info = ret.file
                }
                CheckStatus.NO_FILE_FRAG_CHUNK -> {
                }
            }
            ResponseEntity.ok(ret)
        } else {
            ResponseEntity.status(204).body(ret)
        }
    }

    @ApiOperation("上传", notes = "支持小文件上传,大文件分片上传(统一分片)")
    @LogAnnotation(ignore = true)
    @PostMapping
    fun upload(fileMetadata: ZenFileMetadata, file: MultipartFile?): ResponseEntity<*> {
        if (file == null) {
            return ResponseEntity.status(HttpStatus.NOT_ACCEPTABLE.value()).body(null)
        }
        val uploadResult = try {
            UploadUtil.upload(fileMetadata, file)
        } catch (e: ZenException) {
            null
        }

        return if (uploadResult != null) {
            when (uploadResult.status) {
            // 单文件完成,插入数据库
                UploadStatus.UPLOADED -> {
                    uploadResult.info = uploadResult.file
                }
            }
            ResponseEntity.ok(uploadResult)
        } else {
            ResponseEntity.status(HttpStatus.NOT_ACCEPTABLE.value()).body(null)
        }
    }
}